# Slow lag counter
tab.lag.DOF <- function(data,fast=NULL,dt=NULL)
{
  t <- data$t
  # intelligently select algorithm
  n <- length(t)
  if(is.null(fast))
  {
    if(n<100) { fast <- FALSE }
    else { fast <- TRUE }
  }
  
  # calculate lag,n(lag) vectors
  if(fast)
  {
    lag.DOF <- variogram.fast(data,dt=dt,CI="IID") # count all lag pairs
    lag.DOF$SVF <- NULL
    
    # lag==0 should not be doubled
    lag.DOF$DOF[1] <- lag.DOF$DOF[1]/2 
  }
  else
  {
    lag <- outer(t,t,FUN="-")
    lag <- abs(lag) # may have to change far in the future
    lag <- c(lag) # collapse to 1D array
    
    # can we think of a faster sorter?
    r <- lag # to be the remainder
    lag <- NULL
    DOF <- NULL
    while(length(r)>0)
    {
      n <- length(r)
      lag <- c(lag,r[1])
      r <- r[r!=r[1]]
      DOF <- c(DOF,n-length(r))
    }
    
    lag.DOF <- list(lag=lag,DOF=DOF)
  }
  
  return(lag.DOF)
}

##################################
# Bandwidth optimizer
#lag.DOF is an unsupported option for end users
akde.bandwidth <- function(data,model,fast=NULL,dt=NULL,lag.DOF=NULL)
{
  if(is.null(lag.DOF)) { lag.DOF <- tab.lag.DOF(data,fast=fast,dt=dt) }
  # Extract lag data
  DOF <- lag.DOF$DOF
  lag <- lag.DOF$lag
    
  tau <- model$tau
  K <- length(tau)
  
  # standardized SVF
  if(K==0 || tau==0)
  {
    g <- Vectorize( function(t) { if(t==0) {0} else {1} } )
  }
  else if(K==1)
  {
    g <- Vectorize( function(t) { 1-exp(-t/tau) } )
  }
  else if(K==2)
  {
    g <- Vectorize( function(t) { 1-(tau[1]*exp(-t/tau[1])-tau[2]*exp(-t/tau[2]))/(tau[1]-tau[2]) } )
  }
  
  n <- length(data$t)
  
  # Mean Integrated Square Error modulo a constant
  MISE <- function(h)
  {
    if(h<=0) {Inf}
    else { (1/n^2)*sum(DOF/(2*g(lag)+2*h^2)) - 2/(2+h^2) + 1/2 }
  }
  
  h <- 1/n^(1/6) # User Silverman's rule of thumb to place lower bound
  h <- optimize(f=MISE,interval=c(h/2,2))$minimum
  
  H <- h^2*model$sigma
  
  rownames(H) <- c("x","y")
  colnames(H) <- c("x","y")
  
  return(H)
}

#######################################
# wrap the kde function for our telemetry data format and CIs.
akde <- function(data,model,alpha=0.05,fast=NULL,dt=NULL,error=0.01)
{
  pb <- txtProgressBar(min=-1,max=3,initial=-1,style=3)
  
  tau <- model$tau
  K <- length(tau)
  
  sigma <- model$sigma
  # covariance area/scale
  GM.sigma <- sqrt(det(sigma))
  # orientation matrix
  R <- sigma/GM.sigma

  # wrapper function to estimate bandwidth matrix
  # par = c(tau,sigma.GM)
  fn.H <- function(par)
  {
    model.par <- model
    if(K>0) { model.par$tau <- par[1:K] }
    model.par$sigma <- par[K+1]*R
    H <- akde.bandwidth(data=data,model=model.par,fast=fast,dt=dt)
    return(H)
  }
  # ML optimal bandwidth
  H <- fn.H(c(tau,GM.sigma))
  
  # wrapper function to estimate bandwidth area
  # par = c(tau,sigma.GM)
  fn.GM.H <- function(par)
  {
    H <- fn.H(par)
    GM.H <- sqrt(det(H))
    return(GM.H)
  }
  # How strongly optimal bandwidth varries with parameter estimates
  d.GM.H <- numDeriv::grad(fn.GM.H,c(tau,GM.sigma))
  
  # ML propagated curvature covariance
  COV <- d.GM.H %*% model$COV.tau %*% d.GM.H
  # effective DOF derived from ML curvature
  GM.H <- sqrt(det(H))
  # confidence intervals from chi^2
  GM.H <- chisq.ci(GM.H,COV,alpha)
    
  # data formatted for ks::kde
  x <- cbind(data$x,data$y)
  
  # object to store crap
  KDE <- list(kde.lower=0,kde.ML=0,kde.upper=0)
  
  setTxtProgressBar(pb,0)
  for(i in 1:3)
  {
    H <- GM.H[i]*R
    KDE[[i]] <- kde(data,H,alpha=error)
    setTxtProgressBar(pb,i)
  }
  
  KDE <- c(KDE,list(alpha=alpha))
  
  class(KDE) <- "akde"

  close(pb)
  return(KDE)
}

##################################
# construct my own kde objects
# was using ks-package but it has some bugs
# alpha is the error goal in my total probability
kde <- function(data,H,alpha=0.01)
{
  x <- data$x
  y <- data$y
    
  min.x <- min(x)
  max.x <- max(x)
  
  min.y <- min(y)
  max.y <- max(y)
  
  std.x <- sqrt(H[1,1])
  std.y <- sqrt(H[2,2])
  
  # how far to extend range from data as to ensure alpha significance in total probability
  z <- sqrt(-2*log(alpha))

  min.x <- min.x - z*std.x
  max.x <- max.x + z*std.x    
    
  min.y <- min.y - z*std.y
  max.y <- max.y + z*std.y    
  
  # how fine to make grid as to ensure alpha significance in total probability
  # ~0.4 calculated from asymptotic behavior of elliptic theta function from Riemmann sum approximation of Gaussian integral
  
  z <- alpha/0.4
  
  dx <- z*std.x
  dy <- z*std.y
  
  mu.x <- (min.x+max.x)/2
  mu.y <- (min.y+max.y)/2
  
  n.x <- ceiling((max.x-min.x)/2/dx)
  n.y <- ceiling((max.y-min.y)/2/dy)
  
  # grid locations
  X <- mu.x + ((-n.x):(n.x))*dx
  Y <- mu.y + ((-n.y):(n.y))*dy
    
  H.inv <- solve(H)

  Gauss <- Vectorize( function(X,Y)
  {
    x <- x-X
    y <- y-Y
    # R takes a shit if I don't expand this spatial product by hand.
    R <- -1/2*H.inv[1,1]*x^2 - H.inv[1,2]*x*y -1/2*H.inv[2,2]*y^2
    R <- mean(exp(R))
    return(R)
  })
   pdf <- outer(X,Y,Gauss) / (2*pi*sqrt(det(H)))

  # This is WAY faster but needs a rotation to handle the x-y correlations fast
#   # do the tensor products by hand...
#   x <- outer(X,x,"-")
#   # carefully note the ordering and signs from here on
#   y <- outer(y,Y,"-")
#   # X-Y grid (ignoring cross correlations for the moment...
#   pdf <- exp(-H.inv[1,1]/2*x^2) %*% exp(-H.inv[2,2]/2*y^2) / (length(data$x)*2*pi*sqrt(det(H)))

  result <- list(x=X,y=Y,pdf=pdf,H=H,dA=dx*dy)
  class(result) <- "KDE"
  
  return(result)
}