#########################################
# Coarce into telemetry object
#########################################
telemetry <- function(data,proj=NULL)
{
  
}


as.telemetry <- function(data,proj=NULL,...)
{
  type <- class(data)
  if (type=="data.frame") { data <- telemetry.data.frame(data,proj) }
  else if (type=="character") { data <- telemetry.csv(data,proj,...) }
  # else if(type=="Move" || type=="MoveStack") { data <- telemetry.move(data) }
  
  return(data)
}


# this assumes a MoveBank data.frame
telemetry.data.frame <- function(csv,proj=NULL)
{
  csv <- data.frame(id=csv$individual.local.identifier,t=csv$timestamp,lon=csv$location.long,lat=csv$location.lat)
  csv <- na.omit(csv)
  csv$t <- as.POSIXct(csv$t)
  csv$t <- as.numeric(csv$t)
  
  xy <- cbind(csv$lon,csv$lat)
  colnames(xy) <- c("x","y")
  
  if(is.null(proj)) { proj <- suggest.proj(csv) }
  xy <- rgdal::project(xy,proj)
  
  csv$x <- xy[,1]
  csv$y <- xy[,2]
  
  id <- levels(csv$id)  
  n <- length(id)
  
  telist <- list()  
  for(i in 1:n)
  {
    telist[[i]] <- csv[csv$id==id[i],]
    telist[[i]]$id <- NULL
    telist[[i]] <- c(list(id=id[i],proj=proj),as.list(telist[[i]]))
    class(telist[[i]]) <- "telemetry"
  }
  names(telist) <- id
  
  if (n>1) { return(telist) }
  else { return(telist[[1]]) }
}


# read in a MoveBank csv file
telemetry.csv <- function(file,proj=NULL,...)
{
  data <- read.csv(file,...)
  data <- telemetry.data.frame(data,proj=proj)
  return(data)
}


# # I need to make sure this is always using lon lat
# telemetry.move <- function(move)
# {
#   if(class(move)=="MoveStack")
#   {
#     move <- move::split(move)
#     track <- lapply(move,
#                 function(x){
#                 Ltr <- list(t=as.numeric(timestamps(x)),lon=as.numeric(coordinates(x)[,1]),lat=as.numeric(coordinates(x)[,2]))
#                 attr(Ltr, "row.names") <- row.names(x)
#                 attr(Ltr, "class") <- "telemetry"
#                 return(Ltr)}
#               )
#   }
#   else
#   {
#     track <- list(t=as.numeric(timestamps(move)),lon=coordinates(move)[,1],lat=coordinates(move)[,2])
#     attr(track, "row.names") <- row.names(move)
#     attr(track, "class") <- "telemetry"
#   } 
#   return(track)
# }


########################################
# Suggest a good projection
########################################
suggest.proj <- function(data,datum="WGS84")
{
  # assume Movebank data.frame
  lon <- data$lon
  lat <- data$lat
  
  # as a first approximation use one-point equidistant at mean geolocation
  lon_0 <- mean(lon)
  lat_0 <- mean(lat)
  proj <- paste("+proj=aeqd +lon_0=",lon_0," +lat_0=",lat_0," +datum=",datum,sep="")
  xy <- rgdal::project(cbind(lon,lat),proj)
  
  # calculate and detrend mean
  mu <- c(mean(xy[,1]),mean(xy[,2]))
  xy <- xy - mu
  colnames(xy) <- c("x","y")
  
  # cross correlation
  cov <- mean(xy[,1]*xy[,2])
  # covariance matrix
  cov <- rbind( c( mean(xy[,1]^2) , cov ) , c( cov , mean(xy[,2]^2) ) )
  
  # figure out long axis (always first dim)
  R <- eigen(cov)$vectors
  # rotate data to long axis
  xy <- xy %*% R
  
  # bi-modal split of data
  xy1 <- xy[xy[,1]<0,]
  xy2 <- xy[xy[,1]>0,]
  
  # bi-modal modes
  mu1 <- c(mean(xy1[,1]),mean(xy1[,2]))
  mu2 <- c(mean(xy2[,1]),mean(xy2[,2]))
  
  # reverse rotation
  R <- solve(R)
  mu1 <- mu1 %*% R
  mu2 <- mu2 %*% R
  
  # re-trend mean
  mu1 <- mu1 + mu
  mu2 <- mu2 + mu
  
  # get long lat
  mu1 <- rgdal::project(mu1,proj,inv=TRUE)[1,]
  mu2 <- rgdal::project(mu2,proj,inv=TRUE)[1,]
  
  if(mu1[1] > mu2[1])
  {
    mu <- mu1
    mu1 <- mu2
    mu2 <- mu
  }
  
  proj <- paste("+proj=tpeqd +lon_1=",mu1[1]," +lat_1=",mu1[2]," +lon_2=",mu2[1]," +lat_2=",mu2[2]," +datum=",datum,sep="")
  
  return(proj)
}

################################
# ZOOM INTO TELEMETRY DATA
telemetry.zoom <- function(...)
{
  manipulate::manipulate(
  { plot.telemetry(...,fraction=fraction) },
  fraction = manipulate::slider(0, 1.0,initial=1)
  ) 
}


#######################################
# PLOT TELEMETRY DATA
#######################################
plot.telemetry <- function(data,model=NULL,akde=NULL,alpha.HR=0.05,alpha=0.05,density=TRUE,col.data="red",col.density="blue",col.contour="black",col.grid="grey",add=FALSE,fraction=1,...)
{
  # adjustments to CI-CIs to make them a bit diminished relatively
  trans <- c(0.5,1,0.5)
  lwd <- c(1,2,1)
  
  # listify everything for generality
  if(class(data)=="telemetry") { data <- list(data)  }
  if(!is.null(model)) { if(class(model)=="ctmm") { model <- list(model) } }
  if(!is.null(akde)) { if(class(akde)=="akde") { akde <- list(akde) } }
  
  n <- length(data)
  
  dist.name <- "meters"
  dist.scale <- 1
  if(!add)
  {
    # bounding locations from data
    min.x <- min(sapply(data, function(d){ min(d$x) } ))
    max.x <- max(sapply(data, function(d){ max(d$x) } ))
    
    min.y <- min(sapply(data, function(d){ min(d$y) } ))
    max.y <- max(sapply(data, function(d){ max(d$y) } ))
    
    # bounding locations from akdes
    if(!is.null(akde))
    {
      min.x <- min(c(min.x, sapply(akde, function(a) { sapply(a[1:3], function(k) { k$x[1] }) })))
      max.x <- max(c(max.x, sapply(akde, function(a) { sapply(a[1:3], function(k) { last(k$x) }) })))
      
      min.y <- min(c(min.y, sapply(akde, function(a) { sapply(a[1:3], function(k) { k$y[1] }) })))
      max.y <- max(c(max.y, sapply(akde, function(a) { sapply(a[1:3], function(k) { last(k$y) }) })))
    }
    
    # bounding locations from Gaussian ctmm
    if(!is.null(model))
    {
      z <- sqrt(-2*log(alpha.HR))
      
      for(i in 1:length(model))
      {
        # proportionality constants for outer CIs
        sigma <- model[[i]]$sigma
        if(length(sigma)==1) { sigma <- sigma*diag(2) }
        
        # capture outer contour if present
        const <- 1
        if(!is.null(model[[i]]$COV.tau))
        {
          K <- length(model[[i]]$tau)
          const <- confint.ctmm(model[[i]],alpha)[K+1,3]/sqrt(det(sigma))
        }

        buff <- z*sqrt(const*diag(sigma))
        
        min.x <- min(min.x, model[[i]]$mu[1] - buff[1])
        max.x <- max(max.x, model[[i]]$mu[1] + buff[1])
        
        min.y <- min(min.y, model[[i]]$mu[2] - buff[2])
        max.y <- max(max.y, model[[i]]$mu[2] + buff[2])
      }
    }
        
    # bounding box
    mu <- c(min.x+max.x,min.y+max.y)/2
    buff <- c(max.x-min.x,max.y-min.y)/2
    
    # now zoom in/out to some fraction of the grid
    buff <- fraction*buff
    
    min.x <- mu[1] - buff[1]
    max.x <- mu[1] + buff[1]
    
    min.y <- mu[2] - buff[2]
    max.y <- mu[2] + buff[2]
    
    # Get best unit scale
    dist <- unit(abs(c(min.x,max.x,min.y,max.y)),"length")
    dist.name <- dist$name
    dist.scale <- dist$scale
    
    xlab <- paste("x ", "(", dist.name, ")", sep="")
    ylab <- paste("y ", "(", dist.name, ")", sep="")
    
    mu <- mu/dist.scale
    
    min.x <- min.x/dist.scale
    max.x <- max.x/dist.scale
    
    min.y <- min.y/dist.scale
    max.y <- max.y/dist.scale
    
    # fix base plot layer
    plot(c(min.x,max.x),c(min.y,max.y), xlab=xlab, ylab=ylab, col=rgb(1,1,1,0), asp=1,...)
  }
  
  #########################
  # PLOT GAUSSIAN CONTOURS
  if(!is.null(model))
  {
    # number of model objects
    n <- length(model)
    
    # contours colour
    col.contour <- array(col.contour,n)
    col.density <- array(col.density,n)
    
    for(i in 1:n)
    {
      tau <- model[[i]]$tau
      K <- length(tau)
      
      # scale coordinates
      model[[i]]$mu <- model[[i]]$mu/dist.scale
      model[[i]]$sigma <- model[[i]]$sigma/dist.scale^2
      
      if(length(model[[i]]$sigma)==1) { model[[i]]$sigma <- model[[i]]$sigma*diag(2) }
      
      # plot ML density function
      if(density)
      {
        # look how lazy I am
        pdf <- kde(list(x=model[[i]]$mu[1],y=model[[i]]$mu[2]),H=model[[i]]$sigma)
        plot.density(pdf,col=col.density[[i]],...)
      }
      
      # plot ML estimate, regular style
      plot.ctmm(model[[i]],alpha.HR,col=col.contour[[i]],lwd=lwd[2],...)
      
      # plot CIs dashed if present
      if(!is.null(model[[i]]$COV.tau))
      {
        model[[i]]$COV.tau <- model[[i]]$COV.tau/dist.scale^4 # don't care about tau, just sigma
        
        # proportionality constants for outer CIs
        const <- confint.ctmm(model[[i]],alpha)[K+1,c(1,3)]/sqrt(det(model[[i]]$sigma))
        
        for(j in 1:2)
        {
          model[[i]]$sigma <- const[j]*model[[i]]$sigma
          plot.ctmm(model[[i]],alpha.HR,col=translucent(col.contour[[i]],trans[1]),...)
        }
      }
    
    }
        
  }

  ##########################################
  # PLOT akde CONTOURS... AND DENSITY
  if(!is.null(akde))
  {
    # number of akde objects
    n <- length(akde)
    
    # contours colour
    col.contour <- array(col.contour,n)
    col.density <- array(col.density,n)
    
    # UNIT CONVERSIONS
    for(i in 1:n)
    {
      for(j in 1:3)
      {
        # unit conversion
        akde[[i]][[j]]$x <- akde[[i]][[j]]$x / dist.scale
        akde[[i]][[j]]$y <- akde[[i]][[j]]$y / dist.scale
        akde[[i]][[j]]$pdf <- akde[[i]][[j]]$pdf * dist.scale^2
        akde[[i]][[j]]$dA <- akde[[i]][[j]]$dA / dist.scale^2
        akde[[i]][[j]]$H <- akde[[i]][[j]]$H / dist.scale^2
      }
      
      # ML DENSITY PLOT
      if(density) { plot.density(akde[[i]]$kde.ML,col=col.density[[i]],...) }
    }
    
    # CONTOURS
    for(i in 1:n)
    {
      for(j in 1:3)
      {
        plot.kde(akde[[i]][[j]],alpha=alpha.HR,col=translucent(col.contour[[i]],trans),lwd=lwd,...)
      }
    }
    
    # RESOLUTION GRID
    if(!add)
    {
      dx <- sqrt(max(sapply( akde , function(a) { a$kde.ML$H[1,1] } )))
      dy <- sqrt(max(sapply( akde , function(a) { a$kde.ML$H[2,2] } )))
      
      # grid points per half
      gp <- ceiling(buff/c(dx,dy))
      col.grid <- translucent(col.grid,0.5)
      abline(v=(mu[1]+i*dx*(-gp[1]:gp[1])), col=col.grid)
      abline(h=(mu[2]+i*dy*(-gp[2]:gp[2])), col=col.grid)
    }
  }
  
  #########################
  # PLOT TELEMETRY DATA
  
  # color array for plots
  col.data <- array(col.data,n)
  
  # automagic the plot point size
  p <- sum(sapply(data, function(d) { length(d$t) } ))
  cex <- 1
  if(p>1000) { cex <- 1000/p }
  
  for(i in 1:n)
  {    
    points(data[[i]]$x/dist.scale, data[[i]]$y/dist.scale, cex=cex, col=col.data[[i]],...)
  }
  
}

##################################
# plot PDF stored as KDE object
plot.density <- function(KDE,col="blue",...)
{
  col <- translucent(col,alpha=(0:255)/255)
  zlim <- c(0,max(KDE$pdf))
  image(KDE$x,KDE$y,KDE$pdf,useRaster=TRUE,zlim=zlim,col=col,add=TRUE,...)
}


#############################
# Plot a KDE object's contours
plot.kde <- function(KDE,alpha=0.05,col="black",...)
{
  # here we make a lookup table to find the pdf @ a given cumulative probability from the max pdf
  pdf <- c(KDE$pdf)
  pdf <- sort(pdf,decreasing=TRUE,method="quick")
  
  cdf <- cumsum(pdf)*KDE$dA
  
  # probability density of contour... there are faster algorithms to search ordered lists
  n <- length(cdf[cdf<=(1-alpha)])
  p <- pdf[n]

  contour(x=KDE$x,y=KDE$y,z=KDE$pdf,levels=p,labels=round((1-alpha)*100),labelcex=1,col=col,add=TRUE,...)
}


##############################
# Plot Gaussian ctmm contours
plot.ctmm <- function(model,alpha=0.05,col="blue",...)
{
  mu <- model$mu
  sigma <- model$sigma
  
  Eigen <- eigen(sigma)
  std <- sqrt(Eigen$values)
  vec <- Eigen$vectors
  
  z <- sqrt(-2*log(alpha))
  
  num <- 100
  theta <- 2*pi*(0:num)/(num+1)
  Sin <- sin(theta)
  Cos <- cos(theta)
  
  x <- mu[1] + z*(Cos*std[1]*vec[1,1] + Sin*std[2]*vec[1,2])
  y <- mu[2] + z*(Cos*std[1]*vec[2,1] + Sin*std[2]*vec[2,2])
  
  xspline(x, y=y, shape=-1, open=FALSE, border=col, ...)
}