## ----  fig.show='hold'---------------------------------------------------
library(ctmm)
data(buffalo)
billy <- buffalo[[1]]
plot.telemetry(billy)
plot.telemetry(buffalo,col.data=rainbow(length(buffalo),alpha=0.5))

## ----  fig.show='hold'---------------------------------------------------
var <- variogram(billy)
plot.variogram(var,fraction=0.005)
plot.variogram(var,fraction=0.65,alpha=0.5)

## ----  fig.show='hold'---------------------------------------------------
m0 <- ctmm(sigma=23*1000^2) # 23 km^2 in m^2
m1 <- ctmm(sigma=23*1000^2,tau=6*24*60^2) # and 6 days in seconds
plot.variogram(var,model=m0,fraction=0.65,alpha=0.5)
plot.variogram(var,model=m1,fraction=0.65,alpha=0.5)

## ----  fig.show='hold'---------------------------------------------------
m2 <- ctmm(sigma=23*1000^2,tau=c(1*60^2,6*24*60^2)) # and 1 hour in seconds
plot.variogram(var,model=m1,fraction=0.002)
plot.variogram(var,model=m2,fraction=0.002)

## ----  fig.show='hold'---------------------------------------------------
plot.variogram(var,model=m1,fraction=0.65,alpha=0.5)
plot.variogram(var,model=m2,fraction=0.65,alpha=0.5)

## ----  fig.show='hold'---------------------------------------------------
# simulate fake buffalo with the same sampling schedule
willy <- simulate(m2,billy$t)
plot.telemetry(willy)
# now calculate and plot its variogram
var2 <- variogram(willy)
plot.variogram(var2,model=m2,fraction=0.65,alpha=0.5)

## ----  fig.show='hold'---------------------------------------------------
# ARGOS type errors
curve(1+x,0,5,xlab="Short time lag",ylab="Semi-variance",ylim=c(0,6))
points(c(0,0),c(0,1))
# detector array type errors (qualitatively only)
curve((1-exp(-x))/(1-exp(-1/2)),0,1/2,xlab="Short time lag",ylab="Semi-variance",ylim=c(0,6),xlim=c(0,5))
curve(1/2+x,1/2,5,xlab="Short time lag",ylab="Semi-variance",ylim=c(0,6),add=TRUE,xlim=c(0,5))
points(1/2,1)

## ----  fig.show='hold'---------------------------------------------------
M0 <- ctmm.fit(billy,m0)
summary(M0)

## ----  fig.show='hold'---------------------------------------------------
M1 <- ctmm.fit(billy,m1)
summary(M1)

## ----  fig.show='hold'---------------------------------------------------
M2 <- ctmm.fit(billy,m2)
summary(M2)

## ----  fig.show='hold'---------------------------------------------------
TAB <- rbind( c(M0$AICc,M0$DOF.mu) , c(M1$AICc,M1$DOF.mu) , c(M2$AICc,M2$DOF.mu) )
colnames(TAB) <- c("AICc","DOF(mu)")
rownames(TAB) <- c("M0","M1","M2")
TAB

## ----  fig.show='hold'---------------------------------------------------
plot.variogram(var,model=list(M0,M1,M2),col.model=c("black","purple","blue"),fraction=0.65,alpha=0.5)
plot.variogram(var,model=list(M0,M1,M2),col.model=c("black","purple","blue"),fraction=0.002)


