\name{akde}
\alias{akde}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Calculate an autocorrelated kernel density estimate
}
\description{
This function calculates autocorrelated kernel density estimates of different confidence levels from \code{telemetry} data and a continuous-time movement model.
}
\usage{
akde(data,model,alpha=0.05,fast=NULL,dt=NULL,error=0.01)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{data}{ 2D timeseries telemetry data represented as a \code{telemetry} object or data.frame or list with components \code{t}, \code{x}, \code{y} all numeric values. }
  \item{model}{A \code{ctmm} movement model from the output of \code{ctmm.fit}.}
  \item{alpha}{Significance level for when propagating uncertainties in the \code{ctmm} parameter estimates.}
  \item{fast, dt}{Arguments passed to all instances of \code{\link{akde.bandwidth}}.}
  \item{error}{Target error for discretization.}
}

\details{
The grid size and resolution is choosen to prevent probability errors larger than \code{error} on a worst case basis.
For realistic data, the total probability of the density estimate is typically orders of magnitude less erroneous than \code{error}.

The grid size is choosen such that locations nearest the boundary will have at least \code{error} portion of their kernel's probability within an ellipse contained by the grid. For this the grid boundary is choosen to be at least a number of bandwidth standard deviations, \eqn{z}, from all sampled locations according to the 2D Gaussian quantile relation \eqn{z^2 =  -2 \log} \code{error}.

The choice of grid cell size is such that the sum of probabilities along any axis is within error of unity according to \eqn{ 2 \sum_{i=1}^\infty \frac{\exp{-\frac{dz^2}{2}}}{\sqrt{2\pi}} = 1 - } \code{error}. Limiting behavior of the Jacobi theta function then gives a suggested grid resolution of \eqn{dz = } \code{error} \eqn{/ 0.398942\cdots} bandwidth standard deviations.
}

\value{Returns an \code{akde} object: a list with component kernel-density objects \code{kde.lower}, \code{kde.ML}, \code{kde.upper}, and the corresponding significance level \code{alpha} for the optimal bandwidth area. Each kernel-density object contains the sampled grid line locations \code{x} and \code{y}, the probability density function evaluated on the sampled grid locations \code{pdf}, the optimal bandwidth matrix \code{H}, and the area of each grid cell \code{dA}. }

\references{}

\author{ Chris H. Fleming. }
% \note{ }

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{ \code{\link{akde.bandwidth}}, \code{\link{kde}} }

\examples{
# Load package and data
library(ctmm)
data(buffalo)
billy <- buffalo[[1]]

# Fit a continuous-velocity model with tau=c(1 hour,10 days)
m2 <- ctmm(tau=c(60^2,10*24*60^2))
M2 <- ctmm.fit(billy,m2)

# Compute akde object
KD2 <- akde(billy,M2)

# Plot data with AKDE contours and zoom slider
telemetry.zoom(billy,akde=KD2)
}

% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%\keyword{ ~kwd1 }
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line
