\name{plot.telemetry}
\alias{plot.telemetry}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
A plotting method for telemetry objects.
}
\description{
Produces simple plots of \code{telemetry} objects, possibly overlayed with a Gaussian \code{ctmm} movement model or an \code{akde} autocorrelated kernel density estimate.
}
\usage{
plot.telemetry(data,model=NULL,akde=NULL,alpha.HR=0.05,alpha=0.05,density=TRUE,col.data="red",col.density="blue",col.contour="black",col.grid="grey",add=FALSE,fraction=1,...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{data}{ The 2D timeseries data represented as a \code{telemetry} object or list of such objects. }
  \item{model}{A Gaussian \code{ctmm} movement model from the output of \code{ctmm.fit} or list of such objects.}
  \item{akde}{An \code{akde} object from the output of \code{akde} or list of such objects.}
  \item{alpha.HR}{Significance level of \code{ctmm} model or \code{akde} density estimate to be displayed. I.e., \code{alpha.HR=0.50} yields the 50\% core home range.}
  \item{alpha}{ Significant levels placed on the \code{ctmm} Gaussian home-range contour estimates themselves. I.e., the 50\% core home-range area can be estimated with 95\% confidence via \code{alpha=0.05}. }
  \item{density}{Plot the maximum likelihood probability density function if \code{TRUE}.}
  \item{col.data}{Color option for telemetry data. Can be an array.}
  \item{col.density}{Color option for the probability density function. Can be an array.}
  \item{col.contour}{Color option for home-range contours. Can be an array.}
  \item{col.grid}{Color option for the maximum likelihood \code{akde} bandwidth grid.}
  \item{add}{Setting to \code{FALSE} will disable the unit conversions and base layer plot, so that \code{plot.telemetry} can be overlayed atop other outputs more easily.}
  \item{fraction}{Fraction of the data, Gaussian \code{ctmm}, or \code{akde} range to plot, whichever is larger. Used by \code{\link{telemetry.zoom}}.}
  \item{\dots}{ Additional options passed to all instances of \code{plot}.}
}


\details{
Confidence intervals placed on the \code{ctmm} Gaussian home-range contour estimates only represent uncertainty in the area and not uncertainty in the mean location, eccentricity, or orientation angle.

For \code{akde} density estimates, the provided contours only represent the uncertainty in the optimal bandwidth area and their significance level is pre-calculated in the \code{akde} object itself, ignoring the \code{alpha} here.
With \code{akde} estimates, it is also important to note the scale of the bandwidth and, by default, grid cells are plotted with \code{akde} contours such that their length and width matches that of a bandwidth kernels' standard deviation in each direction. Therefore, this grid provides a visual approximation of the kernel-density esitmate's ``resolution''.}

\value{Returns a plot of \eqn{x} vs. \eqn{y}, and, if specified, Gaussian \code{model} or \code{akde} home range. \code{akde} plots also come with a standard resolution grid.}

\references{
C. H. Fleming, J. M. Calabrese, T. Mueller, K.A. Olson, P. Leimgruber, and W. F. Fagan. (2014). From fine-scale foraging to home ranges: A semi-variance approach to identifying movement modes across spatiotemporal scales. \href{http://www.jstor.org/discover/10.1086/675504}{The American Naturalist, 183(5), E154-E167.}

C. H. Fleming and J. M. Calabrese and T. Mueller and K. A. Olson and P. Leimgruber and W. F. Fagan (2014).
Non-Markovian maximum likelihood estimation of autocorrelated movement processes
\href{http://onlinelibrary.wiley.com/doi/10.1111/2041-210X.12176/abstract}{Methods in Ecology and Evolution, 5(5) 462-472}.

C. H. Fleming and W. F. Fagan and T. Mueller and K. A. Olson and P. Leimgruber and J. M. Calabrese (2015).
Rigorous home-range estimation with movement data: A new autocorrelated kernel-density estimator.
\href{http://www.esajournals.org/doi/abs/10.1890/14-2010.1}{Ecology, doi:10.1890/14-2010.1}.
}

\author{C. H. Fleming.}

\note{}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{ \code{\link{akde}}, \code{\link{ctmm.fit}}, \code{\link{plot}}, \code{\link{telemetry.zoom}}.  }

\examples{
# Load package and data
library(ctmm)
data(buffalo)

# Extract movement data for a single animal
billy <- buffalo[[1]]

# Plot the data
plot.telemetry(billy)
}

% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%\keyword{ ~kwd1 }
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line
