\name{plot.variogram}
\alias{plot.variogram}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
A plotting method for variogram objects.
}
\description{
Produces simple plots of \code{varigram} objects (semi-variance vs. time lag) and model semi-variance functions, with approximate confidence interevals around the semi-variance estimates.
}
\usage{
plot.variogram(variogram, model=NULL, alpha=0.05, frac=0.5, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{variogram}{A \code{variogram} object calculated using \code{\link{variogram}}.}
  \item{model}{A \code{ctmm} movement model object in the same format as the output of \code{ctmm.fit} or \code{variogram.fit}.}
  \item{alpha}{Significance level of confidence bands (95\% default CIs). Can be an array.}
  \item{frac}{
The proportion of the variogram object, \code{variogram}, that will be plotted. By convention, half is shown. The tail end is generally garbage.
}
  \item{\dots}{
Additional \code{plot} function parameters.
}
}

\details{ }

\value{Returns a plot of semi-variance vs. time lag, with the empirical variogram in blue and the \code{model} semi-variance function in red if specified.}

\references{
C. H. Fleming, J. M. Calabrese, T. Mueller, K.A. Olson, P. Leimgruber, and W. F. Fagan. (2014). From fine-scale foraging to home ranges: A semi-variance approach to identifying movement modes across spatiotemporal scales. \href{http://www.jstor.org/discover/10.1086/675504}{The American Naturalist, 183(5), E154-E167.}
}
\author{J. M. Calabrese and C. H. Fleming}

\note{The errors of the empirical variogram are correlated. Smooth trends are not necessarily significant.}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{ \code{\link{ctmm.fit}}, \code{\link{plot}}, \code{\link{variogram}}, \code{\link{variogram.fit}}, \code{\link{variogram.zoom}}. }

\examples{
# Load package and data
library(ctmm)
data(buffalo)

# Extract movement data for a single animal
billy <- buffalo[[1]]

# Calculate variogram
v1 <- variogram(billy)

# Plot the variogram
plot.variogram(v1)
}

% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%\keyword{ ~kwd1 }
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line
